/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2017 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <stdio.h>

#include "cmsis_os2.h"                  /* ::CMSIS:RTOS2 */
#include "Board_LED.h"                  /* ::Board Support:LED */
#include "Board_Buttons.h"              /* ::Board Support:Buttons */
#include "Board_ADC.h"                  /* ::Board Support:A/D Converter */

#include "RTE_Components.h"             /* Component selection */
#include CMSIS_device_header

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

volatile int32_t delay_val = 500;

osThreadId_t tid_thrADC;                /* Thread id of thread: ADC */
osThreadId_t tid_thrLED;                /* Thread id of thread: LED */
osThreadId_t tid_thrBUT;                /* Thread id of thread: BUT */

/*----------------------------------------------------------------------------
  thrADC: get ADC value and set delay
 *----------------------------------------------------------------------------*/
void thrADC(void *argument) {

  for (;;) {
    ADC_StartConversion();
    osDelay(20);                                               /* Wait */

    if (ADC_ConversionDone() == 0) {
      delay_val = (ADC_GetValue() >> 4) + 10;                  /* scale delay value */
    }
  }

}

/*----------------------------------------------------------------------------
  thrLED: blink LED
 *----------------------------------------------------------------------------*/
void thrLED(void *argument) {
  uint32_t led_max    = LED_GetCount();
  uint32_t led_num    = 0;

  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_On(led_num);                                           /* Turn specified LED on */
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_Off(led_num);                                          /* Turn specified LED off */

    led_num++;                                                 /* Change LED number */
    if (led_num >= led_max) {
      led_num = 0;                                             /* Restart with first LED */
    }
  }

}

/*----------------------------------------------------------------------------
  thrBUT: check button state
 *----------------------------------------------------------------------------*/
void thrBUT(void *argument) {
  uint32_t button_msk = (1U << Buttons_GetCount()) - 1;

  for (;;) {
    osDelay(delay_val);                                        /* Wait */
    while (Buttons_GetState() & (button_msk));                 /* Wait while holding USER button */
    osThreadFlagsSet(tid_thrLED, 0x0001);
  }

}

/*----------------------------------------------------------------------------
  configure SystemCoreClock:
     System Clock source         PLL (OSC)
     OSC Frequency(Hz)           8000000
     SYSCLK(Hz)                  120000000
     PLLMUL                      30
     PREDIV                      1
 *----------------------------------------------------------------------------*/
void SystemCoreClockConfigure(void) {

  SCG->SOSCCSR = SCG_SOSCCSR_SOSCERR(1);

  /* set up OSC clock. */
  /* Setup dividers. */
  SCG->SOSCDIV = (SCG_SOSCDIV_SOSCDIV1(1) |
                  SCG_SOSCDIV_SOSCDIV2(1)  );

  /* Set OSC configuration. */
  SCG->SOSCCFG = (SCG_SOSCCFG_EREFS(1) |
                  SCG_SOSCCFG_RANGE(2) );

  /* Enable clock. */
  SCG->SOSCCSR = (SCG_SOSCCSR_SOSCEN(1)  |
                  SCG_SOSCCSR_SOSCLPEN(1) );

  /* Wait for OSC clock to be valid. */
  while (!(SCG->SOSCCSR & SCG_SOSCCSR_SOSCVLD(1))) __NOP();

 /* Init SIRC */
  SCG->SIRCCSR = 0U;

  /* Setup dividers. */
  SCG->SIRCDIV = SCG_SIRCDIV_SIRCDIV1(1) | SCG_SIRCDIV_SIRCDIV2(2);

  /* Set SIRC configuration. */
  SCG->SIRCCFG = SCG_SIRCCFG_RANGE(1);

  /* Enable clock. */
  SCG->SIRCCSR = SCG_SIRCCSR_SIRCEN(1);

  /* Wait for SIRC clock to be valid. */
  while (!(SCG->SIRCCSR & SCG_SIRCCSR_SIRCVLD(1))) __NOP();

  /* Change to SIRC as system clock source */
  SCG->RCCR = (SCG_RCCR_DIVSLOW(3) |
               SCG_RCCR_DIVBUS(0)  |
               SCG_RCCR_DIVCORE(0) |
               SCG_RCCR_SCS(2)     );

  /* Wait for clock source switch finished. */
  while ((SCG->RCCR & 0x0F000000) != 0x02000000) __NOP();;

  /* Init Firc */
  SCG->FIRCCSR = 0U;

  /* Setup dividers. */
  SCG->FIRCDIV = (SCG_FIRCDIV_FIRCDIV1(1) |    /* ADC0 uses FIR as clock source therefore enable FIR clock output */
                  SCG_FIRCDIV_FIRCDIV2(1)  );

  /* Set FIRC configuration. */
  SCG->FIRCCFG = SCG_FIRCCFG_RANGE(0);

  /* Enable clock. */
  SCG->FIRCCSR |= SCG_FIRCCSR_FIRCEN(1);

  /* Wait for FIRC clock to be valid. */
  while (!(SCG->FIRCCSR & SCG_FIRCCSR_FIRCVLD(1))) __NOP();;

  /* Init SysPll */
  SCG->SPLLCSR = SCG_SPLLCSR_SPLLERR(1);

  /* Setup dividers. */
  SCG->SPLLDIV = (SCG_SPLLDIV_SPLLDIV1(1) |
                  SCG_SPLLDIV_SPLLDIV2(2)  );

  /* Set PLL configuration. */
  SCG->SPLLCFG = (SCG_SPLLCFG_SOURCE(0) |           /* osc */
                  SCG_SPLLCFG_PREDIV(0) |           /* prediv 1 */
                  SCG_SPLLCFG_MULT(14)   );         /* pll multiply 30 */

  /* Enable clock. */
  SCG->SPLLCSR = SCG_SPLLCSR_SPLLEN(1);

  /* Wait for PLL clock to be valid. */
  while (!(SCG->SPLLCSR & SCG_SPLLCSR_SPLLVLD(1))) __NOP();

  /* Change to use PLL as system clock source */
  SCG->RCCR = (SCG_RCCR_DIVSLOW(4) |
               SCG_RCCR_DIVBUS(1)  |
               SCG_RCCR_DIVCORE(0) |
               SCG_RCCR_SCS(6)     );

  /* Wait for clock source switch finished. */
  while ((SCG->RCCR & 0x0F000000) != 0x06000000) __NOP();
}

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {
 
  tid_thrBUT = osThreadNew (thrBUT, NULL, NULL);               /* create BUT thread */
  tid_thrLED = osThreadNew (thrLED, NULL, NULL);               /* create LED thread */
  tid_thrADC = osThreadNew (thrADC, NULL, NULL);               /* create ADC thread */

  for (;;) {}
}
 
/*----------------------------------------------------------------------------
  main: create tasks and start kernel
 *----------------------------------------------------------------------------*/
int main (void) {
  uint32_t button_msk = (1U << Buttons_GetCount()) - 1;

  // System Initialization
  SystemCoreClockConfigure();
  SystemCoreClockUpdate();

  LED_Initialize();                                            /* initalize LEDs */
  Buttons_Initialize();                                        /* initalize Buttons */
  ADC_Initialize();                                            /* initialize ADC */

#ifdef RTE_Compiler_EventRecorder
  // Initialize and start Event Recorder
  EventRecorderInitialize(EventRecordAPI, 1U);
#endif

  osKernelInitialize ();                                       /* initialize CMSIS-RTOS */

  osThreadNew(app_main, NULL, NULL);                           /* create application main thread */

  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                                           /* start thread execution */
  }
}
